@@ -23,15 +23,24 @@ __all__ = (
 )
 
 
+class JsonResponse(HttpResponse):
+    "HttpResponse subclass for returning JSON data."
+
+    def __init__(self, *args, **kwargs):
+        kwargs['content_type'] = 'application/json'
+        super(JsonResponse, self).__init__(*args, **kwargs)
+
+
 class LookupBase(object):
     "Base class for all django-selectable lookups."
 
     form = BaseLookupForm
+    response = JsonResponse
 
     def _name(cls):
         app_name = cls.__module__.split('.')[-2].lower()
         class_name = cls.__name__.lower()
-        name = u'%s-%s' % (app_name, class_name)       
+        name = u'%s-%s' % (app_name, class_name)
         return name
     name = classmethod(_name)
 
@@ -58,18 +67,22 @@ class LookupBase(object):
         raise NotImplemented()
 
     def format_item(self, item):
-         return {
-            'id': conditional_escape(self.get_item_id(item)),
-            'value': conditional_escape(self.get_item_value(item)),
-            'label': conditional_escape(self.get_item_label(item))
+        "Construct result dictionary for the match item."
+        result = {
+            'id': self.get_item_id(item),
+            'value': self.get_item_value(item),
+            'label': self.get_item_label(item),
         }
+        for key in settings.SELECTABLE_ESCAPED_KEYS:
+            if key in result:
+                result[key] = conditional_escape(result[key])
+        return result
 
-    def paginate_results(self, request, results, limit):
-        paginator = Paginator(results, limit)
-        try:
-            page = int(request.GET.get('page', '1'))
-        except ValueError:
-            page = 1
+    def paginate_results(self, results, options):
+        "Return a django.core.paginator.Page of results."
+        limit = options.get('limit', settings.SELECTABLE_MAX_LIMIT)
+        paginator = Paginator(results, limit)        
+        page = options.get('page', 1)
         try:
             results = paginator.page(page)
         except (EmptyPage, InvalidPage):
@@ -77,27 +90,40 @@ class LookupBase(object):
         return results
 
     def results(self, request):
-        data = []
+        "Match results to given term and return the serialized HttpResponse."
+        results = {}
         form = self.form(request.GET)
         if form.is_valid():
-            term = form.cleaned_data.get('term', '')
-            limit = form.cleaned_data.get('limit', None)
+            options = form.cleaned_data
+            term = options.get('term', '')
             raw_data = self.get_query(request, term)
-            page_data = None      
-            if limit:
-                page_data = self.paginate_results(request, raw_data, limit)
-                raw_data = page_data.object_list
-            for item in raw_data:
-                data.append(self.format_item(item))
-            if page_data and hasattr(page_data, 'has_next') and page_data.has_next():
-                data.append({
-                    'id': '',
-                    'value': '',
-                    'label': _('Show more results'),
-                    'page': page_data.next_page_number()
-                })        
-        content = json.dumps(data, cls=DjangoJSONEncoder, ensure_ascii=False)
-        return HttpResponse(content, content_type='application/json')    
+            results = self.format_results(raw_data, options)
+        content = self.serialize_results(results)
+        return self.response(content)
+
+    def format_results(self, raw_data, options):
+        '''
+        Returns a python structure that later gets serialized.
+        raw_data
+            full list of objects matching the search term
+        options
+            a dictionary of the given options
+        '''
+        page_data = self.paginate_results(raw_data, options)
+        results = {}
+        meta = options.copy()
+        meta['more'] = _('Show more results')
+        if page_data and page_data.has_next():
+            meta['next_page'] = page_data.next_page_number()
+        if page_data and page_data.has_previous():
+            meta['prev_page'] = page_data.previous_page_number()
+        results['data'] = map(self.format_item, page_data.object_list)
+        results['meta'] = meta
+        return results
+
+    def serialize_results(self, results):
+        "Returns serialized results for sending via http."
+        return json.dumps(results, cls=DjangoJSONEncoder, ensure_ascii=False)
 
 
 class ModelLookup(LookupBase):
