@@ -86,7 +86,7 @@
                 if (self.allowMultiple) {
                     $(input).val("");
                     $(input).data("autocomplete").term = "";
-                    if ($(self.hiddenMultipleSelector + '[value=' + item.id + ']').length === 0) {
+                    if ($(self.hiddenMultipleSelector + '[value="' + item.id + '"]').length === 0) {
                         var newInput = $('<input />', {
                             'type': 'hidden',
                             'name': self.hiddenName,
@@ -190,7 +190,7 @@
                 if (self.options.formatLabel) {
                     label = self.options.formatLabel(label, item);
                 }
-                if (self.options.highlightMatch) {
+                if (self.options.highlightMatch && this.term) {
                     var re = new RegExp("(?![^&;]+;)(?!<[^<>]*)(" +
                     $.ui.autocomplete.escapeRegex(this.term) +
                     ")(?![^<>]*>)(?![^&;]+;)", "gi");
